# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import pecan
from pecan import rest
import wsmeext.pecan as wsme_pecan

from surveil.api.controllers.v1.datamodel import command
from surveil.common import util


class CommandController(rest.RestController):

    def __init__(self, command_name):
        pecan.request.context['command_name'] = command_name
        self._id = command_name

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose(command.Command)
    def get(self):
        """Returns a specific command."""
        c = pecan.request.mongo_connection.shinken.commands.find_one(
            {"command_name": self._id}
        )
        return command.Command(**c)

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose(None, body=command.Command, status_code=204)
    def put(self, data):
        """Modify this command.

        :param data: a command within the request body.
        """

        command_dict = data.as_dict()
        if "command_name" not in command_dict.keys():
            command_dict['command_name'] = self._id

        pecan.request.mongo_connection.shinken.commands.update(
            {"command_name": self._id},
            command_dict
        )

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose(None, status_code=204)
    def delete(self):
        """Delete this command."""
        pecan.request.mongo_connection.shinken.commands.remove(
            {"command_name": self._id}
        )


class CommandsController(rest.RestController):

    @pecan.expose()
    def _lookup(self, command_id, *remainder):
        return CommandController(command_id), remainder

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose([command.Command])
    def get_all(self):
        """Returns all commands."""
        commands = [c for c
                    in pecan.request.mongo_connection.shinken.commands.find()]

        return [command.Command(**c) for c in commands]

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose(command.Command,
                         body=command.Command,
                         status_code=201)
    def post(self, data):
        """Create a new command.

        :param data: a command within the request body.
        """
        pecan.request.mongo_connection.shinken.commands.insert(
            data.as_dict()
        )


# Copyright 2015 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import pecan
from pecan import rest
import wsme.types as wtypes
import wsmeext.pecan as wsme_pecan

from surveil.api.datamodel.config import realm
from surveil.api.handlers.config import realm_handler
from surveil.common import util


class RealmsController(rest.RestController):

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose([realm.Realm])
    def get_all(self):
        """Returns all realms."""
        handler = realm_handler.RealmHandler(pecan.request)
        realms = handler.get_all()
        return realms

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose(realm.Realm, wtypes.text)
    def get_one(self, realm_name):
        """Returns a specific realm."""
        handler = realm_handler.RealmHandler(pecan.request)
        realm = handler.get(realm_name)
        return realm

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose(body=realm.Realm, status_code=201)
    def post(self, data):
        """Create a new realm.

        :param data: a realm within the request body.
        """
        handler = realm_handler.RealmHandler(pecan.request)
        handler.create(data)

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose(realm.Realm, wtypes.text, status_code=204)
    def delete(self, realm_name):
        """Deletes a specific realm."""
        handler = realm_handler.RealmHandler(pecan.request)
        handler.delete(realm_name)

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose(realm.Realm,
                         wtypes.text,
                         body=realm.Realm,
                         status_code=204)
    def put(self, realm_name, realm):
        """Updates a specific realm."""
        handler = realm_handler.RealmHandler(pecan.request)
        handler.update(realm_name, realm)



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import time

import wsme
import wsme.types as wtypes

from surveil.api.controllers.v1.datamodel import types


class CheckResult(types.Base):
    time_stamp = wsme.wsattr(wtypes.text,
                             mandatory=False,
                             default=str(int(time.time())))
    """The time the check was executed. Defaults to now."""

    return_code = wsme.wsattr(int, mandatory=True)
    """The return code of the check."""

    output = wsme.wsattr(wtypes.text, mandatory=True)
    """The output of the check."""

    @classmethod
    def sample(cls):
        return cls(
            time_stamp="1409087486",
            return_code=0,
            output="CPU Usage 98%|c[cpu]=98%;80;95;0;100"
        )



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from surveil.api.controllers.v2.config import businessimpactmodulations
from surveil.api.controllers.v2.config import checkmodulations
from surveil.api.controllers.v2.config import commands
from surveil.api.controllers.v2.config import contactgroups
from surveil.api.controllers.v2.config import contacts
from surveil.api.controllers.v2.config import hostgroups
from surveil.api.controllers.v2.config import hosts
from surveil.api.controllers.v2.config import macromodulations
from surveil.api.controllers.v2.config import notificationways
from surveil.api.controllers.v2.config import realms
from surveil.api.controllers.v2.config import reload_config
from surveil.api.controllers.v2.config import servicegroup
from surveil.api.controllers.v2.config import services
from surveil.api.controllers.v2.config import timeperiods

from pecan import rest


class ConfigController(rest.RestController):
    """Root config controller."""
    hosts = hosts.HostsController()
    commands = commands.CommandsController()
    services = services.ServicesController()
    reload_config = reload_config.ReloadConfigController()
    contacts = contacts.ContactsController()
    timeperiods = timeperiods.TimePeriodsController()
    realms = realms.RealmsController()
    servicegroups = servicegroup.ServiceGroupsController()
    hostgroups = hostgroups.HostGroupsController()
    contactgroups = contactgroups.ContactGroupsController()
    businessimpactmodulations = (businessimpactmodulations.
                                 BusinessImpactModulationsController())
    notificationways = notificationways.NotificationWaysController()
    checkmodulations = checkmodulations.CheckModulationsController()
    macromodulations = macromodulations.MacroModulationController()
    # engine = EngineController()



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from surveil.api.controllers.v1 import commands
from surveil.api.controllers.v1 import hello
from surveil.api.controllers.v1 import hosts
from surveil.api.controllers.v1 import reload_config
from surveil.api.controllers.v1 import services


class V1Controller(object):
    """Version 1 API controller root."""
    hello = hello.HelloController()
    hosts = hosts.HostsController()
    commands = commands.CommandsController()
    services = services.ServicesController()
    reload_config = reload_config.ReloadConfigController()



# Copyright 2015 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from surveil.api.datamodel.config import businessimpactmodulation
from surveil.api.handlers import handler


class BusinessImpactModulationHandler(handler.Handler):
    """Fulfills a request on the business impact modulation resource."""

    def get(self, name):
        """Return a business impact modulation."""

        t = (self.request.mongo_connection.
             shinken.businessimpactmodulations).find_one(
            {"business_impact_modulation_name": name},
            {'_id': 0}
        )
        return businessimpactmodulation.BusinessImpactModulation(**t)

    def update(self, name, modulation):
        """Modify an existing business impact modulation."""
        modulation_dict = modulation.as_dict()
        if "business_impact_modulation_name" not in modulation_dict.keys():
            modulation_dict['business_impact_modulation_name'] = modulation

        self.request.mongo_connection.shinken.businessimpactmodulations.update(
            {"business_impact_modulation_name": name},
            {"$set": modulation_dict},
            upsert=True
        )

    def delete(self, name):
        """Delete existing business impact modulation."""
        self.request.mongo_connection.shinken.businessimpactmodulations.remove(
            {"business_impact_modulation_name": name}
        )

    def create(self, modulation):
        """Create a new business impact modulation."""
        self.request.mongo_connection.shinken.businessimpactmodulations.insert(
            modulation.as_dict()
        )

    def get_all(self):
        """Return all business impact modulations."""
        modulations = [m for m
                       in self.request.mongo_connection.
                       shinken.businessimpactmodulations.find(
                           {"register": {"$ne": "0"}},
                           {'_id': 0}
                       )]
        modulations = [businessimpactmodulation.BusinessImpactModulation(**m)
                       for m in modulations]
        return modulations



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import copy
import json

from surveil.api.datamodel.config import command
from surveil.tests.api import functionalTest


class TestCommandController(functionalTest.FunctionalTest):

    def setUp(self):
        super(TestCommandController, self).setUp()
        self.commands = [
            {u"command_name": u"check_test1",
             u"command_line": u"/test/test1/test.py"},
            {u"command_name": u"check_test2",
             u"command_line": u"/test/test2/test.py"},
        ]
        self.mongoconnection.shinken.commands.insert(
            copy.deepcopy(self.commands)
        )

    def test_get_all_commands(self):
        response = self.get('/v2/config/commands')

        self.assert_count_equal_backport(
            json.loads(response.body.decode()),
            self.commands
        )
        self.assertEqual(response.status_int, 200)

    def test_get_specific_command(self):

        response = self.get('/v2/config/commands/check_test2')

        self.assert_count_equal_backport(
            json.loads(response.body.decode()),
            self.commands[1]
        )
        self.assertEqual(response.status_int, 200)

    def test_update_command(self):
        put_body = {"command_line": "test_put",
                    "command_name": "check_test2"}
        response = self.put_json(
            "/v2/config/commands/check_test2", params=put_body
        )

        expected_command = {u"command_name": u"check_test2",
                            u"command_line": u"test_put"}

        mongo_command = command.Command(
            **self.mongoconnection.shinken.commands.find_one(
                {'command_name': 'check_test2'}
            )
        )

        self.assertEqual(expected_command, mongo_command.as_dict())
        self.assertEqual(response.status_int, 204)

    def test_delete_command(self):
        response = self.delete('/v2/config/commands/check_test2')

        expected_commands = [
            {u"command_name": u"check_test1",
             u"command_line": u"/test/test1/test.py"}
        ]
        mongo_commands = [command.Command(**c).as_dict() for c
                          in self.mongoconnection.shinken.commands.find()]

        self.assertEqual(expected_commands, mongo_commands)
        self.assertEqual(response.status_int, 204)

    def test_add_command(self):
        new_command = {
            "command_name": "newcommand",
            "command_line": "/usr/bin/newcommand -hello"
        }
        response = self.post_json(
            "/v2/config/commands",
            params=new_command
        )

        commands = [command.Command(**c).as_dict() for c
                    in self.mongoconnection.shinken.commands.find()]

        self.assertTrue(new_command in commands)
        self.assertEqual(response.status_int, 201)



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from surveil.api.datamodel.config import macromodulation
from surveil.api.handlers import handler


class MacroModulationHandler(handler.Handler):
    """Fulfills a request on the macro modulation resource."""

    def get(self, modulation_name):
        """Return a macro modulation."""

        m = self.request.mongo_connection.shinken.macromodulations.find_one(
            {"macromodulation_name": modulation_name}, {'_id': 0}
        )
        return macromodulation.MacroModulation(**m)

    def update(self, modulation_name, modulation):
        """Modify an existing macro modulation."""
        modulation_dict = modulation.as_dict()
        if "macromodulation_name" not in modulation_dict.keys():
            modulation_dict['contactgroup_name'] = modulation_name

        self.request.mongo_connection.shinken.macromodulations.update(
            {"macromodulation_name": modulation_name},
            {"$set": modulation_dict},
            upsert=True
        )

    def delete(self, modulation_name):
        """Delete existing macro modulation."""
        self.request.mongo_connection.shinken.macromodulations.remove(
            {"macromodulation_name": modulation_name}
        )

    def create(self, modulation):
        """Create a new macro modulation."""
        self.request.mongo_connection.shinken.macromodulations.insert(
            modulation.as_dict()
        )

    def get_all(self):
        """Return all macro modulation objects."""
        modulations = [m for m
                       in self.request.mongo_connection.
                       shinken.macromodulations.find(
                           # Don't return templates
                           {
                               "register": {"$ne": "0"}
                           },
                           {
                               "_id": 0
                           }
                       )
                       ]

        modulations = [macromodulation.MacroModulation(**m)
                       for m in modulations]

        return modulations



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import wsme
import wsme.types as wtypes

from surveil.api.controllers.v1.datamodel import types


class Host(types.Base):
    host_name = wsme.wsattr(wtypes.text, mandatory=True)
    """The name of the host"""

    address = wsme.wsattr(wtypes.text, mandatory=True)
    """The address of the host. Normally, this is an IP address."""

    max_check_attempts = wsme.wsattr(int, mandatory=False, default=3)

    check_period = wsme.wsattr(wtypes.text, mandatory=False, default='24x7')
    """The time period during which active checks of this host can be made."""

    contacts = wsme.wsattr(wtypes.text, mandatory=False, default='')
    """A list of the short names of the contacts that should be notified."""

    contact_groups = wsme.wsattr(wtypes.text, mandatory=False, default='')
    """List of the short names of the contact groups that should be notified"""

    notification_interval = wsme.wsattr(int, mandatory=False, default=30)

    notification_period = wsme.wsattr(wtypes.text, mandatory=False,
                                      default='24x7')

    use = wsme.wsattr(wtypes.text, mandatory=False)
    """The template to use for this host"""

    # TODO(aviau): Custom fields starting without '_' should raise an error.
    custom_fields = wsme.wsattr(
        wtypes.DictType(wtypes.text, wtypes.text),
        mandatory=False
    )
    """Custom fields for the host"""

    def __init__(self, **kwargs):
        super(Host, self).__init__(**kwargs)

        # Custom fields start with '_'. Detect them ans assign them.
        custom_fields = [i for i in kwargs.items()
                         if isinstance(i[0], str) and i[0][0] == '_']
        if len(custom_fields) > 0:
            self.custom_fields = {}
            for item in custom_fields:
                self.custom_fields[item[0]] = item[1]

    def as_dict(self):
        host_dict = super(Host, self).as_dict()
        custom_fields = host_dict.pop("custom_fields", None)
        if custom_fields:
            for item in custom_fields.items():
                host_dict[item[0]] = item[1]
        return host_dict

    @classmethod
    def sample(cls):
        return cls(
            use="generic-host",
            host_name="bogus-router",
            address="192.168.1.254",
            max_check_attempts=5,
            check_period="24x7",
            contacts="admin,carl",
            contact_groups="router-admins",
            notification_interval=30,
            notification_period="24x7",
            custom_fields={"OS_AUTH_URL": "http://localhost:8080/v2"}
        )



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""Script to query OpenStack and optionally configure its linked Shinken
with discovered elements/devices from Openstack."""


import argparse
import os
import sys
import urlparse

import keystoneclient.v2_0 as kc
import surveilclient.client as sc


#############################################################################

parser = argparse.ArgumentParser(
    epilog=__doc__,
    formatter_class=argparse.RawTextHelpFormatter,
    # TODO(gst): continue..
)

auth_group_help = '''The Openstack authentification values to use.
One can use the explicit CLI arguments or also use environments variables.
The environment variable to use are the same as the CLI arguments, but
in upper case, without the leading '--' and with others '-' replaced by '_'.
Example:
\t$ export OS_AUTH_URL=http://198.72.123.64:5000/v2.0 survey-cli ..
NB: The CLI arguments take precedence over the environment variables.
'''

auth_group = parser.add_argument_group('OS_AUTH', description=auth_group_help)

add = auth_group.add_argument

add('-A', '--api-url',
    help="The Surveil API URL (http(s)://HOST(:PORT)/(URI)).")
add('-U', '--os-username', help="The Openstack USERNAME to use for querying.")
add('-P', '--os-password', help="The Openstack PASSWORD to use for querying.")
add('-u', '--os-auth-url', help="The Openstack AUTH_URL to use for querying.")
add('-t', '--os-tenant-name',
    help="The Openstack TENANT_NAME to use for querying.")
add('-r', '--os-region-name', help="The Openstack REGION_NAME to use.")


OS_AUTH_PARAMS = tuple(
    act.dest
    for act in auth_group._actions[1:]
    # skip the first one which is the group help itself.
)

#############################################################################


def parse_args(args=None):

    cfg = parser.parse_args(args)

    for param in OS_AUTH_PARAMS:
        value = getattr(cfg, param, None)
        if value is None:
            setattr(cfg, param, os.environ.get(param.upper()))

    return cfg


def main(args=None):

    cfg = parse_args(args)

    # Make the inventory of the OpenStack install
    keystone = kc.Client(
        username=cfg.os_username,
        tenant_name=cfg.os_tenant_name,
        password=cfg.os_password,
        auth_url=cfg.os_auth_url,
    )

    endpoints = keystone.service_catalog.get_endpoints()

    cli_surveil = sc.Client(cfg.api_url, version='1_0')

    for ep in endpoints.get('identity', []):

        cli_surveil.hosts.create(
            host_name='OS_keystone_host_%s' % ep['id'],
            use='linux-keystone',
            address=urlparse.urlparse(ep['publicURL']).hostname,
            custom_fields={
                '_OS_AUTH_URL': ep['publicURL'],
                '_OS_USERNAME': cfg.os_username,
                '_OS_PASSWORD': cfg.os_password,
                '_OS_TENANT_NAME': cfg.os_tenant_name,
                '_KS_SERVICES': 'identity',
            }
        )

    for ep in endpoints.get('image', []):
        cli_surveil.hosts.create(
            host_name='OS_glance_host_%s' % ep['id'],
            use='linux-glance',
            address=urlparse.urlparse(ep['publicURL']).hostname,
            custom_fields={
                '_OS_AUTH_URL': ep['publicURL'],
                '_OS_USERNAME': cfg.os_username,
                '_OS_PASSWORD': cfg.os_password,
                '_OS_TENANT_NAME': cfg.os_tenant_name,
                '_OS_GLANCE_URL': ep['publicURL'] + '/v1',
            }
        )

    # Reload the surveil config
    cli_surveil.reload_config()

if __name__ == '__main__':
    sys.exit(main())



# Copyright 2015 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import pecan
from pecan import rest
import requests
import wsmeext.pecan as wsme_pecan

from surveil.api.datamodel.actions import downtime
from surveil.api.datamodel import info
from surveil.common import util


class DowntimeController(rest.RestController):

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose(info.Info,
                         body=downtime.Downtime,
                         status_code=200)
    def post(self, dt):
        """Put a host/service in downtime."""

        data = dt.as_dict()
        data.update({'action': 'add'})

        requests.post(
            pecan.request.ws_arbiter_url + "/downtime",
            data=data
        )

        return info.Info(message='Downtime received.')

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose(info.Info,
                         body=downtime.Downtime,
                         status_code=200)
    def delete(self, dt):
        """Delete a host/service downtime."""

        data = dt.as_dict()
        data.update({'action': 'delete'})

        requests.post(
            pecan.request.ws_arbiter_url + "/downtime",
            data=data
        )

        return info.Info(message='Downtime received.')



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import copy
import json


from surveil.tests.api import functionalTest


class TestMacroModulationController(functionalTest.FunctionalTest):

    def setUp(self):
        super(TestMacroModulationController, self).setUp()
        self.modulations = [
            {
                'macromodulation_name': 'HighDuringNight',
                'modulation_period': 'night',
                '_CRITICAL': 20,
                '_WARNING': 10,
            },
            {
                'macromodulation_name': 'LowDuringNight',
                'modulation_period': 'night',
                '_CRITICAL': 10,
                '_WARNING': 20,
            }
        ]
        self.mongoconnection.shinken.macromodulations.insert(
            copy.deepcopy(self.modulations)
        )

    def test_get_all_macromodulations(self):
        response = self.get('/v2/config/macromodulations')

        self.assert_count_equal_backport(
            [
                {
                    'macromodulation_name': 'HighDuringNight',
                    'modulation_period': 'night',
                    'macros': {
                        '_CRITICAL': 20,
                        '_WARNING': 10}},
                {
                    'macromodulation_name': 'LowDuringNight',
                    'modulation_period': 'night',
                    'macros': {
                        '_CRITICAL': 10,
                        '_WARNING': 20}}
            ],
            json.loads(response.body.decode())

        )
        self.assertEqual(response.status_int, 200)

    def test_get_one_macromodulation(self):
        response = self.get('/v2/config/macromodulations/HighDuringNight')

        self.assertEqual(
            json.loads(response.body.decode()),
            {'macromodulation_name': 'HighDuringNight',
             'modulation_period': 'night',
             'macros': {
                 '_CRITICAL': 20,
                 '_WARNING': 10}}
        )

    def test_create_macromodulation(self):
        m = {
            'macromodulation_name': 'TEST_CREATE_MODULATION',
            'modulation_period': 'night',
            'macros': {
                '_CRITICAL': 10,
                '_WARNING': 20
            }
        }

        self.post_json('/v2/config/macromodulations', m)

        self.assertIsNotNone(
            self.mongoconnection.shinken.macromodulations.find_one(
                {
                    'macromodulation_name': 'TEST_CREATE_MODULATION',
                    '_CRITICAL': 10,
                    '_WARNING': 20
                }
            )
        )

    def test_delete_macromodulation(self):
        self.assertIsNotNone(
            self.mongoconnection.shinken.macromodulations.find_one(
                {"macromodulation_name": 'HighDuringNight'}
            )
        )

        self.delete('/v2/config/macromodulations/HighDuringNight')

        self.assertIsNone(
            self.mongoconnection.shinken.macromodulations.find_one(
                {"macromodulation_name": 'HighDuringNight'}
            )
        )

    def test_put_macromodulation(self):
        self.assertEqual(
            self.mongoconnection.shinken.macromodulations.find_one(
                {'macromodulation_name': 'HighDuringNight'}
            )['modulation_period'],
            'night'
        )

        self.put_json(
            '/v2/config/macromodulations/HighDuringNight',
            {"macromodulation_name": "HighDuringNight",
             "modulation_period": "TESTUPDATE"}
        )

        self.assertEqual(
            self.mongoconnection.shinken.macromodulations.find_one(
                {'macromodulation_name': 'HighDuringNight'}
            )['modulation_period'],
            'TESTUPDATE'
        )



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from __future__ import with_statement

import setuptools


description = 'An Alignak plugin'
long_description = (''' .. ''')


setuptools.setup(
    name='check_example',
    version="1.0",
    packages=setuptools.find_packages(),
    author="Vincent Fournier",
    author_email="vincent.fournier@savoirfairelinux.com",
    long_description=long_description,
    description=description,
    platforms=['any'],
    install_requires=[],
    entry_points="""
    [console_scripts]
    check_example = check_example.check_example:main
    """,
)



# Copyright 2015 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import copy
import json

from surveil.tests.api import functionalTest


class TestConfigController(functionalTest.FunctionalTest):

    def setUp(self):
        super(TestConfigController, self).setUp()
        self.config = [
            {
                "user_name": "bob",
                "config": {"k": "v",
                           "k2": "v2"}
            }
        ]
        self.mongoconnection.surveil.bansho.config.insert(
            copy.deepcopy(self.config)
        )

    def test_get_post_get(self):

        # At first, conf is empty
        self.assertIsNone(
            self.mongoconnection.surveil.bansho.config.find_one(
                {"user_name": "surveil-default-user"}
            )
        )
        response = self.get('/v2/bansho/config')
        self.assertEqual({}, json.loads(response.body.decode()))

        # Now, post config
        config = {"key": "val",
                  "morekey": "moreval"}
        self.post_json('/v2/bansho/config', params=config)

        # Now config is what we gave to the API
        response = self.get('/v2/bansho/config')
        self.assertEqual(config, json.loads(response.body.decode()))
        self.assertIsNotNone(
            self.mongoconnection.surveil.bansho.config.find_one(
                {"user_name": "surveil-default-user"}
            )
        )



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import pecan
from pecan import rest
import requests
import wsmeext.pecan as wsme_pecan

from surveil.api.controllers.v2 import logs
from surveil.api.datamodel import checkresult
from surveil.api.datamodel.status import live_host
from surveil.api.datamodel.status import live_query
from surveil.api.datamodel.status import live_service
from surveil.api.datamodel.status.metrics import live_metric
from surveil.api.datamodel.status.metrics import time_delta
from surveil.api.handlers.status import live_host_handler
from surveil.api.handlers.status import live_service_handler
from surveil.api.handlers.status.metrics import live_metric_handler
from surveil.common import util


class HostsController(rest.RestController):

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose([live_host.LiveHost])
    def get_all(self):
        """Returns all hosts."""
        handler = live_host_handler.HostHandler(pecan.request)
        hosts = handler.get_all()
        return hosts

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose([live_host.LiveHost], body=live_query.LiveQuery)
    def post(self, query):
        """Given a LiveQuery, returns all matching hosts."""
        handler = live_host_handler.HostHandler(pecan.request)
        hosts = handler.get_all(live_query=query)
        return hosts

    @pecan.expose()
    def _lookup(self, host_name, *remainder):
        return HostController(host_name), remainder


class ConfigController(rest.RestController):

    @util.policy_enforce(['authenticated'])
    @pecan.expose()
    def get_all(self):
        """Returns config from a specific host."""
        return "Dump CONFIG"


class HostServicesController(rest.RestController):

    @pecan.expose()
    def _lookup(self, service_name, *remainder):
        return HostServiceController(service_name), remainder


class HostServiceMetricsController(rest.RestController):

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose([live_metric.LiveMetric])
    def get(self):
        """Returns all metrics name for a host with a service."""
        handler = live_metric_handler.MetricHandler(pecan.request)
        metrics_name = handler.get_metric(
            pecan.request.context['host_name'],
            pecan.request.context['service_name']
        )
        return metrics_name

    @pecan.expose()
    def _lookup(self, metric_name, *remainder):
        return HostServiceMetricController(metric_name), remainder


class HostMetricsController(rest.RestController):

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose([live_metric.LiveMetric])
    def get(self):
        """Returns all metrics name for a host."""
        handler = live_metric_handler.MetricHandler(pecan.request)
        metrics_name = handler.get_metric(pecan.request.context['host_name'])
        return metrics_name

    @pecan.expose()
    def _lookup(self, metric_name, *remainder):
        return HostMetricController(metric_name), remainder


class HostCheckResultsSubController(rest.RestController):

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose(body=checkresult.CheckResult, status_code=204)
    def post(self, data):
        """Submit a new check result.

        :param data: a check result within the request body.
        """
        result = data.as_dict()
        result['host_name'] = pecan.request.context['host_name']

        requests.post(
            pecan.request.ws_arbiter_url + "/push_check_result",
            data=result
        )


class ServiceCheckResultsSubController(rest.RestController):

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose(body=checkresult.CheckResult, status_code=204)
    def post(self, data):
        """Submit a new check result.

        :param data: a check result within the request body.
        """
        result = data.as_dict()
        result['host_name'] = pecan.request.context['host_name']

        result['service_description'] = pecan.request.context[
            'service_name'
        ]

        requests.post(
            pecan.request.ws_arbiter_url + "/push_check_result",
            data=result
        )


class HostServiceController(rest.RestController):

    metrics = HostServiceMetricsController()
    results = ServiceCheckResultsSubController()

    def __init__(self, service_name):
        pecan.request.context['service_name'] = service_name
        self.service_name = service_name

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose(live_service.LiveService)
    def get(self):
        """Returns a specific host service."""
        handler = live_service_handler.ServiceHandler(pecan.request)
        service = handler.get(
            pecan.request.context['host_name'],
            self.service_name
        )
        return service


class HostServiceMetricController(rest.RestController):

    def __init__(self, metric_name):
        pecan.request.context['metric_name'] = metric_name
        self.metric_name = metric_name

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose(live_metric.LiveMetric)
    def get(self):
        """Return the last measure for the metric name

            of the service name on the host name.
            """
        handler = live_metric_handler.MetricHandler(pecan.request)
        metric = handler.get(
            self.metric_name,
            pecan.request.context['host_name'],
            pecan.request.context['service_name']
        )
        return metric

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose([live_metric.LiveMetric], body=time_delta.TimeDelta)
    def post(self, time):
        """Given a LiveQuery, returns all matching s."""
        handler = live_metric_handler.MetricHandler(pecan.request)
        metrics = handler.get_all(time_delta=time,
                                  metric_name=self.metric_name,
                                  host_name=pecan.request.context['host_name'],
                                  service_description=pecan.request.
                                  context['service_name'])
        return metrics


class HostMetricController(rest.RestController):

    def __init__(self, metric_name):
        pecan.request.context['metric_name'] = metric_name
        self.metric_name = metric_name

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose(live_metric.LiveMetric)
    def get(self):
        """Return the last measure for the metric name

        of the service name on the host name
        """
        handler = live_metric_handler.MetricHandler(pecan.request)
        metric = handler.get(
            self.metric_name,
            pecan.request.context['host_name']
        )
        return metric

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose([live_metric.LiveMetric], body=time_delta.TimeDelta)
    def post(self, time):
        """Given a LiveQuery, returns all matching s."""
        handler = live_metric_handler.MetricHandler(pecan.request)
        metrics = handler.get_all(time_delta=time,
                                  metric_name=self.metric_name,
                                  host_name=pecan.request.context['host_name']
                                  )
        return metrics


class HostController(rest.RestController):

    services = HostServicesController()
    # See init for controller creation. We need host_name to instanciate it
    # externalcommands = ExternalCommandsController()
    # config = config.ConfigController()
    events = logs.LogsController()
    metrics = HostMetricsController()
    results = HostCheckResultsSubController()

    def __init__(self, host_name):
        pecan.request.context['host_name'] = host_name
        self.host_name = host_name

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose(live_host.LiveHost)
    def get(self):
        """Returns a specific host."""
        handler = live_host_handler.HostHandler(pecan.request)
        host = handler.get(self.host_name)
        return host



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from surveil.api.datamodel.config import checkmodulation
from surveil.api.handlers import handler


class CheckModulationHandler(handler.Handler):
    """Fulfills a request on the check modulation resource."""

    def get(self, checkmodulation_name):
        """Return a check modulation."""
        c = self.request.mongo_connection.shinken.checkmodulations.find_one(
            {"checkmodulation_name": checkmodulation_name}
        )
        return checkmodulation.CheckModulation(**c)

    def update(self, checkmodulation_name, checkmodulation):
        """Modify existing check modulation."""
        checkmodulation_dict = checkmodulation.as_dict()
        if "checkmodulation_name" not in checkmodulation_dict.keys():
            checkmodulation_dict['checkmodulation_name'] = checkmodulation_name

        self.request.mongo_connection.shinken.checkmodulations.update(
            {"checkmodulation_name": checkmodulation_name},
            {"$set": checkmodulation_dict},
            upsert=True
        )

    def delete(self, checkmodulation_name):
        """Delete an existing check modulation."""
        self.request.mongo_connection.shinken.checkmodulations.remove(
            {"checkmodulation_name": checkmodulation_name}
        )

    def create(self, data):
        """Create a new check modulation."""
        self.request.mongo_connection.shinken.checkmodulations.insert(
            data.as_dict()
        )

    def get_all(self):
        """Return all check modulation."""
        find = self.request.mongo_connection.shinken.checkmodulations.find()
        checkmodulations = [c for c in find]

        return [checkmodulation.CheckModulation(**c) for c in checkmodulations]



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from __future__ import unicode_literals

import os

from paste import deploy
import pecan


def get_config_filename():
    abspath = os.path.abspath(__file__)
    path = os.path.dirname(abspath)
    filename = "config.py"
    return os.path.join(path, filename)


def get_pecan_config():
    # Set up the pecan configuration
    return pecan.configuration.conf_from_file(get_config_filename())


def setup_app(pecan_config):
    app_conf = dict(pecan_config.app)

    app = pecan.make_app(
        app_conf.pop('root'),
        logging=getattr(pecan_config, 'logging', {}),
        **app_conf
    )

    return app


def load_app():
    return deploy.loadapp('config:/etc/surveil/api_paste.ini')


def app_factory(global_config, **local_conf):
    return VersionSelectorApplication()


class VersionSelectorApplication(object):
    def __init__(self):
        pc = get_pecan_config()

        self.v1 = setup_app(pecan_config=pc)
        self.v2 = setup_app(pecan_config=pc)

    def __call__(self, environ, start_response):
        if environ['PATH_INFO'].startswith('/v1/'):
            return self.v1(environ, start_response)
        return self.v2(environ, start_response)



# Copyright 2015 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import pecan
from pecan import rest
import wsme.types as wtypes
import wsmeext.pecan as wsme_pecan

from surveil.api.datamodel.config import contactgroup
from surveil.api.handlers.config import contactgroup_handler
from surveil.common import util


class ContactGroupsController(rest.RestController):

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose([contactgroup.ContactGroup])
    def get_all(self):
        """Returns all contact groups."""
        handler = contactgroup_handler.ContactGroupHandler(pecan.request)
        contact_groups = handler.get_all()
        return contact_groups

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose(contactgroup.ContactGroup, wtypes.text)
    def get_one(self, group_name):
        """Returns a contact group."""
        handler = contactgroup_handler.ContactGroupHandler(pecan.request)
        contactgroup = handler.get(group_name)
        return contactgroup

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose(body=contactgroup.ContactGroup, status_code=201)
    def post(self, data):
        """Create a new contact group.

        :param data: a contact group within the request body.
        """
        handler = contactgroup_handler.ContactGroupHandler(pecan.request)
        handler.create(data)

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose(contactgroup.ContactGroup, wtypes.text,
                         status_code=204)
    def delete(self, group_name):
        """Delete a specific contact group."""
        handler = contactgroup_handler.ContactGroupHandler(pecan.request)
        handler.delete(group_name)

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose(contactgroup.ContactGroup,
                         wtypes.text,
                         body=contactgroup.ContactGroup,
                         status_code=204)
    def put(self, group_name, contactgroup):
        """Update a specific contact group."""
        handler = contactgroup_handler.ContactGroupHandler(pecan.request)
        handler.update(group_name, contactgroup)



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import copy
import json

from surveil.api.datamodel.config import host
from surveil.api.datamodel.config import service
from surveil.tests.api import functionalTest


class TestHostController(functionalTest.FunctionalTest):

    def setUp(self):
        super(TestHostController, self).setUp()
        self.hosts = [
            {
                "host_name": "bogus-router", "address": "192.168.1.254",
                "max_check_attempts": 5, "check_period": "24x7",
                "contacts": "admin,carl", "contact_groups": "router-admins",
                "notification_interval": 30, "notification_period": "24x7",
                "_CRITICAL": "10"
            },
            {
                "host_name": "bogus-router2", "address": "192.168.1.254",
                "max_check_attempts": 5, "check_period": "24x7",
                "contacts": "admin,carl", "contact_groups": "router-admins",
                "notification_interval": 30, "notification_period": "24x7"
            },
            {
                "host_name": "bogus-router333", "address": "192.168.1.254",
                "max_check_attempts": 5, "check_period": "24x7",
                "contacts": "admin,carl", "contact_groups": "router-admins",
                "notification_interval": 30, "notification_period": "24x7",
                'use': 'test'
            },
        ]
        self.mongoconnection.shinken.hosts.insert(
            copy.deepcopy(self.hosts)
        )

        self.services = [
            {
                "host_name": "bogus-router",
                "service_description": "service-example",
                "check_command": "check-disk!/dev/sdb1",
                "max_check_attempts": 5,
                "check_interval": 5,
                "retry_interval": 3,
                "check_period": "24x7",
                "notification_interval": 30,
                "notification_period": "24x7",
                "contacts": "surveil-ptl,surveil-bob",
                "contact_groups": "linux-admins"
            }
        ]
        self.mongoconnection.shinken.services.insert(
            copy.deepcopy(self.services)
        )

    def test_get_all_hosts(self):
        response = self.get('/v2/config/hosts')

        # Adjust self.host content to reflect custom_fields sub-dict
        c_fields = {}
        for h in self.hosts:
            if '_CRITICAL' in h.keys():
                c_fields['_CRITICAL'] = h['_CRITICAL']
                h.pop('_CRITICAL')
                h['custom_fields'] = c_fields

        self.assert_count_equal_backport(
            json.loads(response.body.decode()),
            self.hosts
        )
        self.assertEqual(response.status_int, 200)

    def test_get_all_hosts_no_templates(self):
        self.mongoconnection.shinken.hosts.insert(
            copy.deepcopy(
                {"host_name": "bogus-router", "address": "192.168.1.254",
                 "max_check_attempts": 5, "check_period": "24x7",
                 "contacts": "admin,carl", "contact_groups": "router-admins",
                 "notification_interval": 30, "notification_period": "24x7",
                 "register": "0"}
            )
        )
        response = self.get('/v2/config/hosts')

        # Adjust self.host content to reflect custom_fields sub-dict
        c_fields = {}
        for h in self.hosts:
            if '_CRITICAL' in h.keys():
                c_fields['_CRITICAL'] = h['_CRITICAL']
                h.pop('_CRITICAL')
                h['custom_fields'] = c_fields

        self.assert_count_equal_backport(
            json.loads(response.body.decode()),
            self.hosts
        )
        self.assertEqual(response.status_int, 200)

    def test_get_specific_host(self):
        response = self.get('/v2/config/hosts/bogus-router333')

        self.assert_count_equal_backport(
            json.loads(response.body.decode()),
            self.hosts[2]
        )
        self.assertEqual(response.status_int, 200)

    def test_get_specific_host_custom_field(self):
        response = self.get('/v2/config/hosts/bogus-router')
        my_host = json.loads(response.body.decode())

        self.assertIn("custom_fields", my_host.keys())
        self.assertNotIn("_CRITICAL", my_host.keys())
        self.assertIsNone(my_host.get("_CRITICAL"))

    def test_update_host(self):
        put_host = {
            u'host_name': u'bogus-router333',
            u'contacts': u'newcontacts',
        }
        response = self.put_json(
            "/v2/config/hosts/bogus-router333", params=put_host
        )

        mongo_host = host.Host(
            **self.mongoconnection.shinken.hosts.find_one(
                {'host_name': 'bogus-router333'}, {'_id': 0}
            )
        )

        expected = {
            'address': u'192.168.1.254',
            'check_period': u'24x7',
            'notification_interval': 30,
            'contacts': u'newcontacts',
            'notification_period': u'24x7',
            'contact_groups': u'',
            'host_name': u'bogus-router333',
            'max_check_attempts': 3,
            'use': u'test'
        }

        self.assertEqual(expected, mongo_host.as_dict())
        self.assertEqual(response.status_int, 204)

    def test_delete_host(self):
        response = self.delete('/v2/config/hosts/bogus-router')

        mongo_hosts = [host.Host(**h) for h
                       in self.mongoconnection.shinken.hosts.find()]

        self.assertEqual(2, len(mongo_hosts))
        self.assertEqual(response.status_int, 204)

    def test_add_host(self):
        new_host = {
            "host_name": "testpost",
            "address": "192.168.1.254",
            "max_check_attempts": 5,
            "check_period": "24x7",
            "contacts": "admin,carl",
            "contact_groups": "router-admins",
            "notification_interval": 3,
            "notification_period": "24x7"
        }
        response = self.post_json("/v2/config/hosts", params=new_host)

        hosts = [host.Host(**h).as_dict() for h
                 in self.mongoconnection.shinken.hosts.find(None, {'_id': 0})]

        self.assertTrue(new_host in hosts)
        self.assertEqual(response.status_int, 201)

    def test_add_host_custom_fields(self):
        my_host = {
            "host_name": "custom_field_host", "address": "192.168.1.254",
            "max_check_attempts": 5, "check_period": "24x7",
            "contacts": "admin,carl", "contact_groups": "router-admins",
            "notification_interval": 30, "notification_period": "24x7",
            "_TEST_CUSTOM_FIELD": "10"
        }

        self.mongoconnection.shinken.hosts.insert(my_host)
        mongo_host = self.mongoconnection.shinken.hosts.find_one(
            {"host_name": "custom_field_host"}
        )
        # In-MongoDB representation should hold custom fields similarly to
        # Shinken:
        #
        # define host {
        #       _CUSTOM     value
        # }
        # (no "custom_fields" sub-dict)
        self.assertNotIn("custom_fields", mongo_host.keys())
        self.assertIn("_TEST_CUSTOM_FIELD", mongo_host.keys())
        self.assertIsNotNone(mongo_host["_TEST_CUSTOM_FIELD"])

    def test_post_add_host_custom_fields(self):
        my_host = {
            "host_name": "custom_field_host", "address": "192.168.1.254",
            "max_check_attempts": 5, "check_period": "24x7",
            "contacts": "admin,carl", "contact_groups": "router-admins",
            "notification_interval": 30, "notification_period": "24x7",
            "custom_fields": {
                "_TEST_CUSTOM_FIELD": "10"
            }
        }

        self.post_json('/v2/config/hosts', my_host)
        mongo_host = self.mongoconnection.shinken.hosts.find_one(
            {"host_name": "custom_field_host"}
        )
        # In-MongoDB representation should hold custom fields similarly to
        # Shinken:
        #
        # define host {
        #       _CUSTOM     value
        # }
        # (no "custom_fields" sub-dict)
        self.assertNotIn("custom_fields", mongo_host.keys())
        self.assertIn("_TEST_CUSTOM_FIELD", mongo_host.keys())
        self.assertIsNotNone(mongo_host["_TEST_CUSTOM_FIELD"])

    def test_get_associated_services(self):
        response = self.get('/v2/config/hosts/bogus-router/services')

        self.assertEqual(
            self.services,
            json.loads(response.body.decode())
        )

    def test_get_specific_service(self):
        response = self.get(
            '/v2/config/hosts/bogus-router/services/service-example'
        )

        self.assertEqual(
            self.services[0],
            json.loads(response.body.decode())
        )

    def test_delete_specific_service(self):
        mongo_services = [service.Service(**h) for h
                          in self.mongoconnection.shinken.services.find()]
        self.assertEqual(1, len(mongo_services))

        self.delete(
            '/v2/config/hosts/bogus-router/services/service-example'
        )

        mongo_services = [service.Service(**h) for h
                          in self.mongoconnection.shinken.services.find()]

        self.assertEqual(0, len(mongo_services))



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from surveil.api.datamodel.config import timeperiod
from surveil.api.handlers import handler


class TimePeriodHandler(handler.Handler):
    """Fulfills a request on the contact resource."""

    def get(self, timeperiod_name):
        """Return a time period."""

        t = self.request.mongo_connection.shinken.timeperiods.find_one(
            {"timeperiod_name": timeperiod_name}, {'_id': 0}
        )
        return timeperiod.TimePeriod(**t)

    def update(self, timeperiod_name, timeperiod):
        """Modify an existing time period."""
        timeperiod_dict = timeperiod.as_dict()
        if "timeperiod_name" not in timeperiod_dict.keys():
            timeperiod_dict['timeperiod_name'] = timeperiod_name

        self.request.mongo_connection.shinken.timeperiods.update(
            {"timeperiod_name": timeperiod_name},
            {"$set": timeperiod_dict},
            upsert=True
        )

    def delete(self, timeperiod_name):
        """Delete existing time period."""
        self.request.mongo_connection.shinken.timeperiods.remove(
            {"timeperiod_name": timeperiod_name}
        )

    def create(self, timeperiod):
        """Create a new time period."""
        self.request.mongo_connection.shinken.timeperiods.insert(
            timeperiod.as_dict()
        )

    def get_all(self):
        """Return all time periods."""
        timeperiods = [t for t
                       in self.request.mongo_connection.
                       shinken.timeperiods.find(
                           {"register": {"$ne": "0"}},
                           {'_id': 0}
                       )]
        timeperiods = [timeperiod.TimePeriod(**t) for t in timeperiods]
        return timeperiods



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from six.moves import urllib


def safe_quote(s):
    """URL-encode strings that are not already URL-encoded."""
    return urllib.parse.quote(s) if s == urllib.parse.unquote(s) else s


class MiniResp(object):

    def __init__(self, error_message, env, headers=[]):
        # The HEAD method is unique: it must never return a body, even if
        # it reports an error (RFC-2616 clause 9.4). We relieve callers
        # from varying the error responses depending on the method.
        if env['REQUEST_METHOD'] == 'HEAD':
            self.body = ['']
        else:
            self.body = [error_message.encode()]
        self.headers = list(headers)
        self.headers.append(('Content-type', 'text/plain'))



# Copyright 2015 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import pecan
from pecan import rest
import wsme.types as wtypes
import wsmeext.pecan as wsme_pecan

from surveil.api.datamodel.config import notificationway
from surveil.api.handlers.config import notificationway_handler
from surveil.common import util


class NotificationWaysController(rest.RestController):

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose([notificationway.NotificationWay])
    def get_all(self):
        """Returns all notification ways."""
        handler = notificationway_handler.NotificationWayHandler(pecan.request)
        notificationsway = handler.get_all()
        return notificationsway

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose(notificationway.NotificationWay, wtypes.text)
    def get_one(self, notificationway_name):
        """Returns a specific notification way."""
        handler = notificationway_handler.NotificationWayHandler(pecan.request)
        notificationway = handler.get(notificationway_name)
        return notificationway

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose(body=notificationway.NotificationWay, status_code=201)
    def post(self, data):
        """Create a new notification way.

        :param data: a notification way within the request body.
        """
        handler = notificationway_handler.NotificationWayHandler(pecan.request)
        handler.create(data)

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose(
        notificationway.NotificationWay,
        wtypes.text,
        status_code=204
    )
    def delete(self, notificationway_name):
        """Returns a specific notification way."""
        handler = notificationway_handler.NotificationWayHandler(pecan.request)
        handler.delete(notificationway_name)

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose(notificationway.NotificationWay,
                         wtypes.text,
                         body=notificationway.NotificationWay,
                         status_code=204)
    def put(self, notificationway_name, notificationway):
        """Update a specific notification way."""
        handler = notificationway_handler.NotificationWayHandler(pecan.request)
        handler.update(notificationway_name, notificationway)



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json

import wsme
import wsme.types as wtypes

from surveil.api.datamodel import types


class LiveQuery(types.Base):
    """Holds a sample query encoded in json."""

    filters = wsme.wsattr(wtypes.text, mandatory=True)
    "The filter expression encoded in json."

    fields = wsme.wsattr([wtypes.text], mandatory=False)
    "List of fields to include in the response."

    @classmethod
    def sample(cls):
        return cls(
            fields=['host_name', 'last_check'],
            filters=json.dumps({
                "isnot": {
                    "state": ["0", "1"],
                    "host_state": ["2"]
                }
            })
        )



# Copyright 2015 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import wsme
import wsme.types as wtypes

from surveil.api.datamodel import types


class ServiceGroup(types.Base):
    servicegroup_name = wsme.wsattr(wtypes.text, mandatory=True)
    members = wsme.wsattr(wtypes.text, mandatory=True)
    alias = wsme.wsattr(wtypes.text, mandatory=False)
    servicegroup_members = wsme.wsattr(wtypes.text, mandatory=False)
    notes = wsme.wsattr(wtypes.text, mandatory=False)
    notes_url = wsme.wsattr(wtypes.text, mandatory=False)
    action_url = wsme.wsattr(wtypes.text, mandatory=False)

    @classmethod
    def sample(cls):
        return cls(
            servicegroup_name='dbservices',
            alias='Database Services',
            members='ms1,SQL Server,ms1,SQL Serverc Agent,ms1,SQL DTC'
        )
